import { App } from '@next-vue/runtime-core';
import { RootRenderFunction } from '@next-vue/runtime-core';
import { VNode } from '@next-vue/runtime-core';

export declare const createApp: () => App<TestElement>;

declare function createComment(text: string): TestComment;

declare function createElement(tag: string): TestElement;

declare function createText(text: string): TestText;

export declare function dumpOps(): NodeOp[];

declare function insert(child: TestNode, parent: TestElement, ref?: TestNode | null): void;

export declare function logNodeOp(op: NodeOp): void;

export declare function mockWarn(): void;

declare function nextSibling(node: TestNode): TestNode | null;

export declare interface NodeOp {
    type: NodeOpTypes;
    nodeType?: NodeTypes;
    tag?: string;
    text?: string;
    targetNode?: TestNode;
    parentNode?: TestElement;
    refNode?: TestNode | null;
    propKey?: string;
    propPrevValue?: any;
    propNextValue?: any;
}

export declare const nodeOps: {
    insert: typeof insert;
    remove: typeof remove;
    createElement: typeof createElement;
    createText: typeof createText;
    createComment: typeof createComment;
    setText: typeof setText;
    setElementText: typeof setElementText;
    parentNode: typeof parentNode;
    nextSibling: typeof nextSibling;
    querySelector: typeof querySelector;
};

export declare const enum NodeOpTypes {
    CREATE = "create",
    INSERT = "insert",
    REMOVE = "remove",
    SET_TEXT = "setText",
    SET_ELEMENT_TEXT = "setElementText",
    PATCH = "patch"
}

export declare const enum NodeTypes {
    TEXT = "text",
    ELEMENT = "element",
    COMMENT = "comment"
}

declare function parentNode(node: TestNode): TestElement | null;

declare function querySelector(): any;

declare function remove(child: TestNode, logOp?: boolean): void;

export declare const render: RootRenderFunction<TestNode, TestElement>;

export declare function renderToString(vnode: VNode): string;

export declare function resetOps(): void;

export declare function serialize(node: TestNode, indent?: number, depth?: number): string;

export declare function serializeInner(node: TestElement, indent?: number, depth?: number): string;

declare function setElementText(el: TestElement, text: string): void;

declare function setText(node: TestText, text: string): void;

export declare interface TestComment {
    id: number;
    type: NodeTypes.COMMENT;
    parentNode: TestElement | null;
    text: string;
}

export declare interface TestElement {
    id: number;
    type: NodeTypes.ELEMENT;
    parentNode: TestElement | null;
    tag: string;
    children: TestNode[];
    props: Record<string, any>;
    eventListeners: Record<string, Function | Function[]> | null;
}

export declare type TestNode = TestElement | TestText | TestComment;

export declare interface TestText {
    id: number;
    type: NodeTypes.TEXT;
    parentNode: TestElement | null;
    text: string;
}

export declare function triggerEvent(el: TestElement, event: string, payload?: any[]): void;

export * from "@next-vue/runtime-core";

export { }
